@@ -30,10 +30,11 @@ struct _TrackerController
 	GObject parent_instance;
 
 	TrackerIndexingTree *indexing_tree;
+	TrackerMonitor *monitor;
 	TrackerStorage *storage;
 	TrackerConfig *config;
+	GSettings *extractor_settings;
 	TrackerFilesInterface *files_interface;
-	GVolumeMonitor *volume_monitor;
 
 	GDBusProxy *control_proxy;
 	GCancellable *control_proxy_cancellable;
@@ -42,11 +43,9 @@ struct _TrackerController
 	GSList *config_recursive_directories;
 	GSList *config_single_directories;
 
-	guint force_recheck_id;
 	guint volumes_changed_id;
 
 	guint index_removable_devices : 1;
-	guint index_optical_discs : 1;
 };
 
 enum {
@@ -54,6 +53,7 @@ enum {
 	PROP_INDEXING_TREE,
 	PROP_STORAGE,
 	PROP_FILES_INTERFACE,
+	PROP_MONITOR,
 	N_PROPS,
 };
 
@@ -79,6 +79,9 @@ tracker_controller_set_property (GObject      *object,
 	case PROP_FILES_INTERFACE:
 		controller->files_interface = g_value_dup_object (value);
 		break;
+	case PROP_MONITOR:
+		controller->monitor = g_value_dup_object (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -112,6 +115,7 @@ add_indexed_directory (TrackerController     *controller,
                        TrackerDirectoryFlags  flags)
 {
 	g_autofree gchar *path = NULL;
+	TrackerStorageType type;
 
 	path = g_file_get_path (file);
 
@@ -129,31 +133,30 @@ add_indexed_directory (TrackerController     *controller,
 		}
 	}
 
-	g_debug ("  Adding:'%s'", path);
+	type = tracker_storage_get_type_for_file (controller->storage, file);
 
-	if (g_settings_get_boolean (G_SETTINGS (controller->config), "enable-monitors"))
-		flags |= TRACKER_DIRECTORY_FLAG_MONITOR;
+	if ((type & TRACKER_STORAGE_REMOVABLE) != 0)
+		flags |= TRACKER_DIRECTORY_FLAG_IS_VOLUME;
+
+	g_debug ("  Adding:'%s'", path);
 
 	tracker_indexing_tree_add (controller->indexing_tree, file, flags);
 }
 
 static void
-add_removable_or_optical_directory (TrackerController *controller,
-                                    GFile             *mount_file)
+add_removable_directory (TrackerController *controller,
+                         GFile             *mount_file)
 {
 	TrackerDirectoryFlags flags;
 	g_autofree gchar *uri = NULL;
 
 	flags = TRACKER_DIRECTORY_FLAG_RECURSE |
 		TRACKER_DIRECTORY_FLAG_PRESERVE |
-		TRACKER_DIRECTORY_FLAG_PRIORITY;
-
-	if (g_settings_get_boolean (G_SETTINGS (controller->config), "enable-monitors")) {
-		flags |= TRACKER_DIRECTORY_FLAG_MONITOR;
-	}
+		TRACKER_DIRECTORY_FLAG_PRIORITY |
+		TRACKER_DIRECTORY_FLAG_IS_VOLUME;
 
 	uri = g_file_get_uri (mount_file);
-	g_debug ("  Adding removable/optical: '%s'", uri);
+	g_debug ("  Adding removable: '%s'", uri);
 	tracker_indexing_tree_add (controller->indexing_tree,
 				   mount_file,
 				   flags);
@@ -175,9 +178,7 @@ mount_point_added_cb (TrackerController *controller,
 
 	if (removable && !controller->index_removable_devices) {
 		g_debug ("  Not crawling, removable devices disabled in config");
-	} else if (optical && !controller->index_optical_discs) {
-		g_debug ("  Not crawling, optical devices discs disabled in config");
-	} else if (!removable && !optical) {
+	} else if (!removable) {
 		TrackerDirectoryFlags flags;
 		GSList *l;
 
@@ -193,28 +194,24 @@ mount_point_added_cb (TrackerController *controller,
 			flags = TRACKER_DIRECTORY_FLAG_RECURSE |
 				TRACKER_DIRECTORY_FLAG_PRESERVE;
 
-			if (g_settings_get_boolean (G_SETTINGS (controller->config), "enable-monitors")) {
-				flags |= TRACKER_DIRECTORY_FLAG_MONITOR;
-			}
-
 			if (g_file_equal (config_file, mount_point_file) ||
 			    g_file_has_prefix (config_file, mount_point_file)) {
 				/* If the config path is contained inside the mount path,
 				 *  then add the config path to re-check */
 				g_debug ("  Re-check of configured path '%s' needed (recursively)",
 				         (gchar *) l->data);
-				tracker_indexing_tree_add (controller->indexing_tree,
-							   config_file,
-							   flags);
+				add_indexed_directory (controller,
+				                       config_file,
+				                       flags);
 			} else if (g_file_has_prefix (mount_point_file, config_file)) {
 				/* If the mount path is contained inside the config path,
 				 *  then add the mount path to re-check */
 				g_debug ("  Re-check of path '%s' needed (inside configured path '%s')",
 				         mount_point,
 				         (gchar *) l->data);
-				tracker_indexing_tree_add (controller->indexing_tree,
-							   config_file,
-							   flags);
+				add_indexed_directory (controller,
+				                       config_file,
+				                       flags);
 			}
 		}
 
@@ -227,24 +224,19 @@ mount_point_added_cb (TrackerController *controller,
 
 			flags = TRACKER_DIRECTORY_FLAG_NONE;
 
-			if (g_settings_get_boolean (G_SETTINGS (controller->config), "enable-monitors")) {
-				flags |= TRACKER_DIRECTORY_FLAG_MONITOR;
-			}
-
 			config_file = g_file_new_for_path (l->data);
 			if (g_file_equal (config_file, mount_point_file) ||
 			    g_file_has_prefix (config_file, mount_point_file)) {
 				g_debug ("  Re-check of configured path '%s' needed (non-recursively)",
 				         (gchar *) l->data);
-				tracker_indexing_tree_add (controller->indexing_tree,
-							   config_file,
-							   flags);
+				add_indexed_directory (controller,
+				                       config_file,
+				                       flags);
 			}
 		}
 	} else {
-		g_debug ("  Adding directories in removable/optical media to crawler's queue");
-		add_removable_or_optical_directory (controller,
-		                                    mount_point_file);
+		g_debug ("  Adding directories in removable media to crawler's queue");
+		add_removable_directory (controller, mount_point_file);
 	}
 }
 
@@ -262,21 +254,6 @@ mount_point_removed_cb (TrackerController *controller,
 	tracker_indexing_tree_remove (controller->indexing_tree, mount_point_file);
 }
 
-static void
-mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
-                      GMount            *mount,
-                      TrackerController *controller)
-{
-	g_autoptr (GFile) mount_root = NULL;
-	g_autofree gchar *uri = NULL;
-
-	mount_root = g_mount_get_root (mount);
-	uri = g_file_get_uri (mount_root);
-	g_debug ("Pre-unmount requested for '%s'", uri);
-
-	tracker_indexing_tree_remove (controller->indexing_tree, mount_root);
-}
-
 static void
 indexing_tree_update_filter (TrackerIndexingTree *indexing_tree,
 			     TrackerFilterType    filter,
@@ -290,6 +267,18 @@ indexing_tree_update_filter (TrackerIndexingTree *indexing_tree,
 		tracker_indexing_tree_add_filter (indexing_tree, filter, strv[i]);
 }
 
+static void
+text_allowlist_update (TrackerIndexingTree *indexing_tree,
+                       GStrv                strv)
+{
+	int i;
+
+	tracker_indexing_tree_clear_allowed_text_patterns (indexing_tree);
+
+	for (i = 0; strv[i]; i++)
+		tracker_indexing_tree_add_allowed_text_pattern (indexing_tree, strv[i]);
+}
+
 static void
 update_filters (TrackerController *controller)
 {
@@ -312,6 +301,11 @@ update_filters (TrackerController *controller)
 	strv = g_settings_get_strv (G_SETTINGS (controller->config), "ignored-directories-with-content");
 	indexing_tree_update_filter (controller->indexing_tree, TRACKER_FILTER_PARENT_DIRECTORY, strv);
 	g_strfreev (strv);
+
+	/* Allowed text patterns */
+	strv = g_settings_get_strv (controller->extractor_settings, "text-allowlist");
+	text_allowlist_update (controller->indexing_tree, strv);
+	g_strfreev (strv);
 }
 
 static void
@@ -346,7 +340,7 @@ update_directories_from_new_config (TrackerController *controller,
 			 * unmounts.
 			 */
 			tracker_indexing_tree_get_root (controller->indexing_tree,
-							file, &flags);
+			                                file, NULL, &flags);
 
 			if ((flags & TRACKER_DIRECTORY_FLAG_PRESERVE) != 0) {
 				flags &= ~(TRACKER_DIRECTORY_FLAG_PRESERVE);
@@ -367,10 +361,6 @@ update_directories_from_new_config (TrackerController *controller,
 		flags |= TRACKER_DIRECTORY_FLAG_RECURSE;
 	}
 
-	if (g_settings_get_boolean (G_SETTINGS (controller->config), "enable-monitors")) {
-		flags |= TRACKER_DIRECTORY_FLAG_MONITOR;
-	}
-
 	/* Second add directories which are new */
 	for (sl = new_dirs; sl; sl = sl->next) {
 		const gchar *path;
@@ -384,7 +374,7 @@ update_directories_from_new_config (TrackerController *controller,
 			TRACKER_NOTE (CONFIG, g_message ("  Adding directory:'%s'", path));
 
 			file = g_file_new_for_path (path);
-			tracker_indexing_tree_add (controller->indexing_tree, file, flags);
+			add_indexed_directory (controller, file, flags);
 		}
 	}
 }
@@ -427,14 +417,11 @@ index_single_directories_cb (TrackerConfig     *config,
 	controller->config_single_directories = tracker_gslist_copy_with_string_data (new_dirs);
 }
 
-static gboolean
-trigger_recheck_idle_cb (gpointer user_data)
+static void
+tracker_controller_check_all_roots (TrackerController *controller)
 {
-	TrackerController *controller = user_data;
 	GList *roots, *l;
 
-	update_filters (controller);
-
 	roots = tracker_indexing_tree_list_roots (controller->indexing_tree);
 
 	for (l = roots; l; l = l->next)	{
@@ -443,24 +430,24 @@ trigger_recheck_idle_cb (gpointer user_data)
 		tracker_indexing_tree_notify_update (controller->indexing_tree, root, FALSE);
 	}
 
-	controller->force_recheck_id = 0;
 	g_list_free (roots);
-
-	return G_SOURCE_REMOVE;
 }
 
 static void
-trigger_recheck_cb (TrackerConfig     *config,
-                    GParamSpec        *pspec,
-                    TrackerController *controller)
+filter_changed_cb (GSettings         *config,
+                   GParamSpec        *pspec,
+                   TrackerController *controller)
 {
-	TRACKER_NOTE (CONFIG, g_message ("Ignored content related configuration changed, checking index..."));
+	update_filters (controller);
+	tracker_controller_check_all_roots (controller);
+}
 
-	if (controller->force_recheck_id == 0) {
-		/* Set idle so multiple changes in the config lead to one recheck */
-		controller->force_recheck_id =
-			g_idle_add (trigger_recheck_idle_cb, controller);
-	}
+static void
+text_allowlist_changed_cb (TrackerConfig *config,
+                           GParamSpec        *pspec,
+                           TrackerController *controller)
+{
+	tracker_controller_check_all_roots (controller);
 }
 
 static gboolean
@@ -470,23 +457,15 @@ index_volumes_changed_idle (gpointer user_data)
 	GSList *mounts_removed = NULL;
 	GSList *mounts_added = NULL;
 	gboolean new_index_removable_devices;
-	gboolean new_index_optical_discs;
 
 	TRACKER_NOTE (CONFIG, g_message ("Volume related configuration changed, updating..."));
 
-	/* Read new config values. Note that if removable devices is FALSE,
-	 * optical discs will also always be FALSE. */
 	new_index_removable_devices = g_settings_get_boolean (G_SETTINGS (controller->config), "index-removable-devices");
-	new_index_optical_discs = (new_index_removable_devices ?
-	                           g_settings_get_boolean (G_SETTINGS (controller->config), "index-optical-discs") :
-	                           FALSE);
 
 	/* Removable devices config changed? */
 	if (controller->index_removable_devices != new_index_removable_devices) {
 		GSList *m;
 
-		/* Get list of roots for currently mounted removable devices
-		 * (excluding optical) */
 		m = tracker_storage_get_device_roots (controller->storage,
 		                                      TRACKER_STORAGE_REMOVABLE,
 		                                      TRUE);
@@ -507,32 +486,6 @@ index_volumes_changed_idle (gpointer user_data)
 		}
 	}
 
-	/* Optical discs config changed? */
-	if (controller->index_optical_discs != new_index_optical_discs) {
-		GSList *m;
-
-		/* Get list of roots for removable devices (excluding optical) */
-		m = tracker_storage_get_device_roots (controller->storage,
-		                                      TRACKER_STORAGE_REMOVABLE | TRACKER_STORAGE_OPTICAL,
-		                                      TRUE);
-
-		/* Set new config value */
-		controller->index_optical_discs = new_index_optical_discs;
-
-		if (controller->index_optical_discs) {
-			/* If previously not indexing and now indexing, need to re-check
-			 * current mounted volumes, add new monitors and index new files
-			 */
-			mounts_added = g_slist_concat (mounts_added, m);
-		} else {
-			/* If previously indexing and now not indexing, need to re-check
-			 * current mounted volumes, remove monitors and remove all resources
-			 * from the store belonging to a optical disc
-			 */
-			mounts_removed = g_slist_concat (mounts_removed, m);
-		}
-	}
-
 	if (mounts_removed) {
 		GSList *sl;
 
@@ -554,8 +507,7 @@ index_volumes_changed_idle (gpointer user_data)
 			g_autoptr (GFile) mount_point_file = NULL;
 
 			mount_point_file = g_file_new_for_path (sl->data);
-			add_removable_or_optical_directory (controller,
-			                                    mount_point_file);
+			add_removable_directory (controller, mount_point_file);
 		}
 
 		g_slist_free_full (mounts_removed, g_free);
@@ -578,6 +530,22 @@ index_volumes_changed_cb (TrackerConfig     *config,
 	}
 }
 
+static void
+enable_monitors_changed_cb (GSettings         *config,
+                            GParamSpec        *pspec,
+                            TrackerController *controller)
+{
+	gboolean enable;
+
+	enable = g_settings_get_boolean (config, "enable-monitors");
+
+	if (enable != tracker_monitor_get_enabled (controller->monitor)) {
+		tracker_monitor_set_enabled (controller->monitor, enable);
+		if (enable)
+			filter_changed_cb (config, pspec, controller);
+	}
+}
+
 static void
 initialize_from_config (TrackerController *controller)
 {
@@ -593,29 +561,13 @@ initialize_from_config (TrackerController *controller)
 	controller->index_removable_devices =
 		g_settings_get_boolean (G_SETTINGS (controller->config), "index-removable-devices");
 
-	/* Note that if removable devices not indexed, optical discs
-	 * will also never be indexed */
-	controller->index_optical_discs = (controller->index_removable_devices ?
-	                                   g_settings_get_boolean (G_SETTINGS (controller->config), "index-optical-discs") :
-	                                   FALSE);
-
 	if (controller->index_removable_devices) {
-		/* Get list of roots for removable devices (excluding optical) */
+		/* Get list of roots for removable devices */
 		mounts = tracker_storage_get_device_roots (controller->storage,
 		                                           TRACKER_STORAGE_REMOVABLE,
 		                                           TRUE);
 	}
 
-	if (controller->index_optical_discs) {
-		GSList *m;
-
-		/* Get list of roots for removable+optical devices */
-		m = tracker_storage_get_device_roots (controller->storage,
-		                                      TRACKER_STORAGE_OPTICAL | TRACKER_STORAGE_REMOVABLE,
-		                                      TRUE);
-		mounts = g_slist_concat (mounts, m);
-	}
-
 	TRACKER_NOTE (CONFIG, g_message ("Setting up directories to iterate from config (IndexSingleDirectory)"));
 
 	dirs = tracker_config_get_index_single_directories (controller->config);
@@ -661,8 +613,7 @@ initialize_from_config (TrackerController *controller)
 		g_autoptr (GFile) mount_point = NULL;
 
 		mount_point = g_file_new_for_path (l->data);
-		add_removable_or_optical_directory (controller,
-		                                    mount_point);
+		add_removable_directory (controller, mount_point);
 	}
 
 	g_slist_free_full (mounts, g_free);
@@ -720,11 +671,9 @@ update_indexed_files_from_proxy (TrackerController *controller,
 		if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
 			if (!tracker_indexing_tree_file_is_indexable (indexing_tree,
 			                                              file, file_info)) {
-				tracker_indexing_tree_add (indexing_tree,
-				                           file,
-				                           TRACKER_DIRECTORY_FLAG_RECURSE |
-				                           TRACKER_DIRECTORY_FLAG_CHECK_MTIME |
-				                           TRACKER_DIRECTORY_FLAG_MONITOR);
+				add_indexed_directory (controller,
+				                       file,
+				                       TRACKER_DIRECTORY_FLAG_RECURSE);
 				g_ptr_array_add (controller->control_proxy_folders,
 				                 g_object_ref (file));
 			} else {
@@ -778,6 +727,39 @@ on_control_proxy_ready (GObject      *source,
 	update_indexed_files_from_proxy (controller, controller->control_proxy);
 }
 
+static void
+log_config (TrackerConfig *config)
+{
+#ifdef G_ENABLE_DEBUG
+	if (TRACKER_DEBUG_CHECK (CONFIG)) {
+		GSList *dirs, *l;
+
+		g_message ("Indexer options:");
+
+		dirs = tracker_config_get_index_recursive_directories (config);
+		if (dirs)
+			g_message ("  Recursive folders:");
+		for (l = dirs; l; l = l->next)
+			g_message ("    %s\n", (char*) l->data);
+
+		dirs = tracker_config_get_index_recursive_directories (config);
+		if (dirs)
+			g_message ("  Non-recursive folders:");
+		for (l = dirs; l; l = l->next)
+			g_message ("    %s\n", (char*) l->data);
+
+		g_message ("  Index removable volumes: %s",
+		           g_settings_get_boolean (G_SETTINGS (config),
+		                                   "index-removable-devices") ?
+		           "on" : "off");
+		g_message ("  Monitor directories: %s",
+		           g_settings_get_boolean (G_SETTINGS (config),
+		                                   "enable-monitors") ?
+		           "on" : "off");
+	}
+#endif
+}
+
 static void
 tracker_controller_constructed (GObject *object)
 {
@@ -796,41 +778,38 @@ tracker_controller_constructed (GObject *object)
 	                         object,
 	                         G_CONNECT_SWAPPED);
 
-	controller->volume_monitor = g_volume_monitor_get ();
-	g_signal_connect_object (controller->volume_monitor, "mount-pre-unmount",
-	                         G_CALLBACK (mount_pre_unmount_cb),
-	                         object, G_CONNECT_SWAPPED);
-
 	controller->config = tracker_config_new ();
-	g_signal_connect (controller->config, "notify::index-recursive-directories",
+	g_signal_connect (controller->config, "changed::index-recursive-directories",
 	                  G_CALLBACK (index_recursive_directories_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::index-single-directories",
+	g_signal_connect (controller->config, "changed::index-single-directories",
 	                  G_CALLBACK (index_single_directories_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::ignored-directories",
-	                  G_CALLBACK (trigger_recheck_cb),
+	g_signal_connect (controller->config, "changed::ignored-directories",
+	                  G_CALLBACK (filter_changed_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::ignored-directories-with-content",
-	                  G_CALLBACK (trigger_recheck_cb),
+	g_signal_connect (controller->config, "changed::ignored-directories-with-content",
+	                  G_CALLBACK (filter_changed_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::ignored-files",
-	                  G_CALLBACK (trigger_recheck_cb),
+	g_signal_connect (controller->config, "changed::ignored-files",
+	                  G_CALLBACK (filter_changed_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::enable-monitors",
-	                  G_CALLBACK (trigger_recheck_cb),
+	g_signal_connect (controller->config, "changed::enable-monitors",
+	                  G_CALLBACK (enable_monitors_changed_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::index-removable-devices",
+	g_signal_connect (controller->config, "changed::index-removable-devices",
 	                  G_CALLBACK (index_volumes_changed_cb),
 	                  object);
-	g_signal_connect (controller->config, "notify::index-optical-discs",
-	                  G_CALLBACK (index_volumes_changed_cb),
+
+	controller->extractor_settings = g_settings_new ("org.freedesktop.Tracker3.Extract");
+	g_signal_connect (controller->extractor_settings, "changed::text-allowlist",
+	                  G_CALLBACK (text_allowlist_changed_cb),
 	                  object);
 
 	g_dbus_proxy_new_for_bus (TRACKER_IPC_BUS,
 	                          G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
 	                          NULL,
-	                          "org.freedesktop.Tracker3.Miner.Files.Control",
+	                          "org.freedesktop.LocalSearch3.Control",
 	                          "/org/freedesktop/Tracker3/Miner/Files/Proxy",
 	                          "org.freedesktop.Tracker3.Miner.Files.Proxy",
 	                          NULL,
@@ -838,6 +817,8 @@ tracker_controller_constructed (GObject *object)
 	                          controller);
 
 	initialize_from_config (controller);
+
+	log_config (controller->config);
 }
 
 static void
@@ -850,13 +831,12 @@ tracker_controller_finalize (GObject *object)
 	g_clear_object (&controller->control_proxy);
 	g_clear_pointer (&controller->control_proxy_folders, g_ptr_array_unref);
 
-	g_clear_handle_id (&controller->force_recheck_id, g_source_remove);
 	g_clear_handle_id (&controller->volumes_changed_id, g_source_remove);
 
 	g_clear_object (&controller->indexing_tree);
 	g_clear_object (&controller->storage);
 	g_clear_object (&controller->config);
-	g_clear_object (&controller->volume_monitor);
+	g_clear_object (&controller->extractor_settings);
 	g_clear_object (&controller->files_interface);
 
 	g_slist_free_full (controller->config_single_directories, g_free);
@@ -898,6 +878,13 @@ tracker_controller_class_init (TrackerControllerClass *klass)
 		                     G_PARAM_WRITABLE |
 		                     G_PARAM_CONSTRUCT_ONLY |
 		                     G_PARAM_STATIC_STRINGS);
+	props[PROP_MONITOR] =
+		g_param_spec_object ("monitor",
+		                     NULL, NULL,
+		                     TRACKER_TYPE_MONITOR,
+		                     G_PARAM_WRITABLE |
+		                     G_PARAM_CONSTRUCT_ONLY |
+		                     G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties (object_class, N_PROPS, props);
 }
@@ -911,11 +898,13 @@ tracker_controller_init (TrackerController *controller)
 
 TrackerController *
 tracker_controller_new (TrackerIndexingTree   *tree,
+                        TrackerMonitor        *monitor,
                         TrackerStorage        *storage,
                         TrackerFilesInterface *files_interface)
 {
 	return g_object_new (TRACKER_TYPE_CONTROLLER,
 			     "indexing-tree", tree,
+	                     "monitor", monitor,
 	                     "storage", storage,
 	                     "files-interface", files_interface,
 			     NULL);
