@@ -23,6 +23,8 @@
 
 #include "tracker-indexing-tree.h"
 
+#include "tracker-indexing-tree-methods.h"
+
 #include <tracker-common.h>
 
 /**
@@ -41,6 +43,7 @@ typedef struct _FindNodeData FindNodeData;
 struct _NodeData
 {
 	GFile *file;
+	char *id;
 	guint flags;
 	guint shallow : 1;
 	guint removing : 1;
@@ -64,6 +67,8 @@ struct _TrackerIndexingTreePrivate
 {
 	GNode *config_tree;
 	GList *filter_patterns;
+	GUdevClient *udev_client;
+	GList *allowed_text_patterns;
 
 	GFile *root;
 	guint filter_hidden : 1;
@@ -103,6 +108,7 @@ static void
 node_data_free (NodeData *data)
 {
 	g_object_unref (data->file);
+	g_free (data->id);
 	g_slice_free (NodeData, data);
 }
 
@@ -203,6 +209,8 @@ tracker_indexing_tree_constructed (GObject *object)
 	data->shallow = TRUE;
 
 	priv->config_tree = g_node_new (data);
+
+	priv->udev_client = g_udev_client_new (NULL);
 }
 
 static void
@@ -225,6 +233,8 @@ tracker_indexing_tree_finalize (GObject *object)
 	                 NULL);
 	g_node_destroy (priv->config_tree);
 
+	g_clear_object (&priv->udev_client);
+
 	if (priv->root) {
 		g_object_unref (priv->root);
 	}
@@ -600,7 +610,7 @@ tracker_indexing_tree_notify_update (TrackerIndexingTree *tree,
 	g_return_val_if_fail (TRACKER_IS_INDEXING_TREE (tree), FALSE);
 	g_return_val_if_fail (G_IS_FILE (file), FALSE);
 
-	root = tracker_indexing_tree_get_root (tree, file, &flags);
+	root = tracker_indexing_tree_get_root (tree, file, NULL, &flags);
 
 	if (tracker_indexing_tree_file_is_root (tree, file)) {
 		g_signal_emit (tree, signals[DIRECTORY_UPDATED], 0, root);
@@ -800,7 +810,7 @@ tracker_indexing_tree_file_is_indexable (TrackerIndexingTree *tree,
 	g_return_val_if_fail (TRACKER_IS_INDEXING_TREE (tree), FALSE);
 	g_return_val_if_fail (G_IS_FILE (file), FALSE);
 
-	config_file = tracker_indexing_tree_get_root (tree, file, &config_flags);
+	config_file = tracker_indexing_tree_get_root (tree, file, NULL, &config_flags);
 	if (!config_file) {
 		/* Not under an added dir */
 		return FALSE;
@@ -825,11 +835,6 @@ tracker_indexing_tree_file_is_indexable (TrackerIndexingTree *tree,
 		return FALSE;
 	}
 
-	/* FIXME: Shouldn't we only do this for file_type == G_FILE_TYPE_DIRECTORY ? */
-	if (config_flags & TRACKER_DIRECTORY_FLAG_IGNORE) {
-		return FALSE;
-	}
-
 	if (g_file_equal (file, config_file)) {
 		return TRUE;
 	} else {
@@ -943,24 +948,11 @@ tracker_indexing_tree_set_filter_hidden (TrackerIndexingTree *tree,
 	g_object_notify (G_OBJECT (tree), "filter-hidden");
 }
 
-/**
- * tracker_indexing_tree_get_root:
- * @tree: a #TrackerIndexingTree
- * @file: a #GFile
- * @directory_flags: (out): return location for the applying #TrackerDirectoryFlags
- *
- * Returns the #GFile that was previously added through tracker_indexing_tree_add()
- * and would equal or contain @file, or %NULL if none applies.
- *
- * If the return value is non-%NULL, @directory_flags would contain the
- * #TrackerDirectoryFlags applying to @file.
- *
- * Returns: (transfer none): the effective parent in @tree, or %NULL
- **/
 GFile *
-tracker_indexing_tree_get_root (TrackerIndexingTree   *tree,
-                                GFile                 *file,
-                                TrackerDirectoryFlags *directory_flags)
+tracker_indexing_tree_get_root (TrackerIndexingTree    *tree,
+                                GFile                  *file,
+                                const char            **id,
+                                TrackerDirectoryFlags  *directory_flags)
 {
 	TrackerIndexingTreePrivate *priv;
 	NodeData *data;
@@ -986,10 +978,17 @@ tracker_indexing_tree_get_root (TrackerIndexingTree   *tree,
 	    (file == data->file ||
 	     g_file_equal (file, data->file) ||
 	     g_file_has_prefix (file, data->file))) {
-		if (directory_flags) {
-			*directory_flags = data->flags;
+		if (!data->id) {
+			data->id = tracker_indexing_tree_get_root_id (tree,
+			                                              data->file,
+			                                              priv->udev_client);
 		}
 
+		if (id)
+			*id = data->id;
+		if (directory_flags)
+			*directory_flags = data->flags;
+
 		return data->file;
 	}
 
@@ -1110,3 +1109,45 @@ tracker_indexing_tree_list_roots (TrackerIndexingTree *tree)
 	                 &nodes);
 	return nodes;
 }
+
+void
+tracker_indexing_tree_clear_allowed_text_patterns (TrackerIndexingTree *tree)
+{
+	TrackerIndexingTreePrivate *priv = tree->priv;
+
+	g_list_free_full (priv->allowed_text_patterns, (GDestroyNotify) g_pattern_spec_free);
+	priv->allowed_text_patterns = NULL;
+}
+
+void
+tracker_indexing_tree_add_allowed_text_pattern (TrackerIndexingTree *tree,
+                                                const char          *pattern_str)
+{
+	TrackerIndexingTreePrivate *priv = tree->priv;
+
+	priv->allowed_text_patterns =
+		g_list_prepend (priv->allowed_text_patterns,
+		                g_pattern_spec_new (pattern_str));
+}
+
+gboolean
+tracker_indexing_tree_file_has_allowed_text_extension (TrackerIndexingTree *tree,
+                                                       GFile               *file)
+{
+	TrackerIndexingTreePrivate *priv = tree->priv;
+	g_autofree gchar *basename = NULL;
+	GList *l;
+
+	basename = g_file_get_basename (file);
+
+	for (l = priv->allowed_text_patterns; l; l = l->next) {
+#if GLIB_CHECK_VERSION (2, 70, 0)
+		if (g_pattern_spec_match_string (l->data, basename))
+#else
+		if (g_pattern_match_string (l->data, basename))
+#endif
+			return TRUE;
+	}
+
+	return FALSE;
+}
