@@ -384,6 +384,10 @@ class MinerCrawlTest(fixtures.TrackerMinerTest):
 
         # Check everything is fine
         result = self.__get_text_documents()
+        unpacked_result = [r[0] for r in result]
+        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
+        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
+        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
         self.assertEqual(len(result), 3)
 
     def test_10_folder_update(self):
@@ -533,6 +537,157 @@ class MinerCrawlTest(fixtures.TrackerMinerTest):
         result = self.tracker.query(query)
         self.assertEqual(result[0][0], "false")
 
+    def test_16_folder_update_offline(self):
+        """
+        Check that updating a folder deletes the old nfo:Folder
+        """
+
+        directory_uri = self.uri("test-monitored/dir1")
+        document_uri = self.uri("test-monitored/dir1/unrelated.txt")
+        dest_uri = self.uri("test-monitored/dir1/unrelated2.txt")
+        directory = self.path("test-monitored/dir1")
+        document = self.path("test-monitored/dir1/unrelated.txt")
+        dest = self.path("test-monitored/dir1/unrelated2.txt")
+        ie_urn = self.__get_file_urn(directory)
+
+        self.assertResourceExists(ie_urn)
+
+        # Create a dir to overwrite the original first, to guarantee
+        # a different inode.
+        os.makedirs(self.path("zzz"))
+
+        self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
+
+        # Replace the dir
+        shutil.rmtree(directory)
+        shutil.move(self.path("zzz"), directory)
+
+        conn = self.miner_fs.get_sparql_connection()
+
+        # Write document
+        with self.await_document_inserted(document) as resource:
+            with open(document, "w") as f:
+                f.write(DEFAULT_TEXT)
+
+        new_ie_urn = self.__get_file_urn(directory)
+        self.assertNotEqual(new_ie_urn, ie_urn)
+
+        self.assertResourceExists(new_ie_urn)
+        self.assertResourceExists(document_uri)
+
+        # Check the old URN is not a folder anymore
+        query = "ASK { <%s> a nfo:Folder }"
+        result = self.tracker.query(query)
+        self.assertEqual(result, [['false']])
+
+        query = "SELECT ?u { ?u nie:isStoredAs <%s> }" % self.uri(directory)
+        result = self.tracker.query(query)
+        self.assertEqual(len(result), 1)
+        self.assertEqual(result[0][0], new_ie_urn)
+
+        query = "SELECT ?u { <%s> nie:interpretedAs ?u }" % self.uri(directory)
+        result = self.tracker.query(query)
+        self.assertEqual(len(result), 1)
+        self.assertEqual(result[0][0], new_ie_urn)
+
+        # Move document
+        resource_id = self.tracker.get_content_resource_id(document_uri)
+        dest = self.path("test-monitored/dir1/unrelated2.txt")
+        with self.await_document_uri_change(resource_id, document, dest):
+            shutil.move(document, dest)
+
+        self.assertResourceMissing(document_uri)
+        self.assertResourceExists(dest_uri)
+
+    def test_17_move_and_replace(self):
+        """
+        Move a file on top of another, and have another file replace the first
+        """
+        source = self.path("test-monitored/file1.txt");
+        source_urn = self.__get_file_urn(source)
+        dest = self.path("test-monitored/dir1/file2.txt");
+        dest_urn = self.__get_file_urn(dest)
+
+        source_id = self.tracker.get_content_resource_id(self.uri(source))
+
+        # Move file overwriting another
+        with self.await_document_uri_change(source_id, source, dest):
+            shutil.move(source, dest)
+
+        result = self.__get_text_documents()
+        self.assertEqual(len(result), 2)
+        unpacked_result = [r[0] for r in result]
+        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
+
+        # Ensure the file replaced by source changed and matches
+        new_dest_urn = self.__get_file_urn(dest)
+        self.assertNotEqual(dest_urn, new_dest_urn)
+        self.assertEqual(source_urn, new_dest_urn)
+
+        # Write a new document in the former position
+        with self.await_document_inserted(source) as resource:
+            with open(source, "w") as f:
+                f.write("Replacement")
+
+        # Ensure the file created at source location is different
+        new_source_urn = self.__get_file_urn(source)
+        self.assertNotEqual(source_urn, new_source_urn)
+
+    def test_18_move_and_replace_dir(self):
+        """
+        Move a directory, and create another directory replacing the first
+        """
+        source = self.path("test-monitored/dir1");
+        source_urn = self.__get_file_urn(source)
+        dest = self.path("test-monitored/moved");
+
+        with self.await_insert_dir(dest):
+            shutil.move(source, dest)
+
+        with self.await_insert_dir(source):
+            os.mkdir(source)
+
+        new_source_urn = self.__get_file_urn(source)
+        dest_urn = self.__get_file_urn(dest)
+
+        self.assertEqual(dest_urn, source_urn)
+        self.assertNotEqual(new_source_urn, source_urn)
+
+    def test_19_move_and_replace_from_hidden(self):
+        """
+        Create a hidden file, and make it visible. Then repeat.
+        """
+
+        old_text = "I am old!"
+        new_text = "I am new!"
+
+        source = self.path("test-monitored/.hidden.txt");
+        dest = self.path("test-monitored/nothidden.txt");
+
+        with open(source, "w") as f:
+            f.write(old_text)
+        with self.await_document_inserted(dest):
+            shutil.move(source, dest)
+
+        urn = self.__get_file_urn(dest)
+
+        with open(source, "w") as f:
+            f.write(new_text)
+
+        resource_id = self.tracker.get_content_resource_id(self.uri(dest))
+
+        with self.tracker.await_content_update(
+            fixtures.DOCUMENTS_GRAPH,
+            resource_id,
+            f'nie:plainTextContent "{old_text}"',
+            f'nie:plainTextContent "{new_text}"',
+            timeout=cfg.AWAIT_TIMEOUT,
+        ):
+            shutil.move(source, dest)
+
+        new_urn = self.__get_file_urn(dest)
+        self.assertNotEqual(urn, new_urn)
+
 
 class IndexedFolderTest(fixtures.TrackerMinerTest):
     """
@@ -608,7 +763,7 @@ class IndexedFolderTest(fixtures.TrackerMinerTest):
     """
     def test_01_move_between_indexed_folders(self):
         """
-        The precreated files and folders should be there
+        Move a file between indexed folders and check the data source changed
         """
         result = self.__get_text_documents()
         self.assertEqual(len(result), 3)
@@ -633,8 +788,9 @@ class IndexedFolderTest(fixtures.TrackerMinerTest):
         self.assertIn(self.uri("test-monitored/file2.txt"), unpacked_result)
         self.assertIn(self.uri("test-monitored/dir1/file3.txt"), unpacked_result)
 
+        # The file was moved to test-monitored
         datasource2 = self.__get_index_folder(dest);
-        self.assertNotEqual(datasource2, self.uri("test-monitored"))
+        self.assertEqual(datasource2, self.uri("test-monitored"))
         self.assertNotEqual(datasource1, datasource2)
 
 
